varying vec4 fragcolor;
varying vec4 fragcoord;


//==========================================
//Textures
//==========================================
uniform sampler2D texture0; //color
uniform sampler2D texture1; //depth
uniform sampler2D texture2; //normal
uniform sampler2D texture3; //blurred bloom buffer
uniform sampler2D texture4; //unblurred bloom buffer
uniform sampler2D texture5; //caustics frame 1
uniform sampler2D texture6; //blurred DOF texture
uniform sampler2D texture7; //wobble texture / caustics frame 2
uniform sampler2D texture10; //noise texture for effects
uniform sampler2D texture11; //ssao texture


uniform float contrast = 1.0;
uniform float brightness = 1.0;
uniform float saturation = 1.0;

//==========================================
//Automatic uniforms
//==========================================
uniform float apptime;
uniform vec2 buffersize;
uniform vec2 camerarange;
uniform float camerazoom;


include "depthtozposition.frag"


//==========================================
//HDR uniforms
//==========================================

uniform float hdrexposure;

//==========================================
// Fog variables
//==========================================

uniform vec4 fogcolor = vec4(1.0);
uniform vec2 fogrange = vec2(0.1,1000.0);


//==========================================
// DOF variables
//==========================================

uniform vec2 dofnearrange;
uniform vec2 doffarrange;
uniform float dofstrength=1.0;


//==========================================
// Caustics variables
//==========================================
uniform vec4 causticsmappingaxes[4];
uniform float causticsframeinterpolation;
uniform vec4 causticscolor;


//==========================================
// Volumetric light scattering variables
//==========================================

#ifdef LW_GODRAYS
	uniform vec3 screenlightpos;
	uniform vec3 lightvector;
	uniform vec4 raycolor = vec4(1.0);
	//uniform float exposure = 0.35;
	#define exposure 1.0
	#define RAYSAMPLES 64
	#define MAXRAYLENGTH 1.0
#endif


//==========================================
// Motion blur variables
//==========================================

uniform vec2 motionblurvector;
uniform float motionblurstrength = 1.0;
uniform float motionblurdistance;


//==========================================
// SSAO Functions
//==========================================

#ifdef LW_SSAO

	float ao=0;
	#define aointensitybias -0.55

	#define ssaostrength 0.5

	float readDepth( in vec2 coord ) {
		//#ifndef __GLSL_CG_DATA_TYPES
		//	coord.y = 1.0 - coord.y;
		//#endif
		return (2.0 * camerarange.x) / (camerarange.y + camerarange.x - texture2D( texture1, coord ).x * (camerarange.y - camerarange.x)) * camerarange.y;	
	}
	
	float compareDepths( in float depth1, in float depth2 ) {
		float aoCap = 1.0;
		float aoMultiplier=10.0;
		float depthTolerance=0.000;
		float aorange = 100.0;///(camerarange.y-camerarange.x);// units in space the AO effect extends to (this gets divided by the camera far range
		float diff = ( clamp(1.0-(depth1-depth2) / aorange,0.0,1.0) );
		float ao = min(aoCap,max(0.0,depth1-depth2-depthTolerance)) * diff;
		ao *= aoMultiplier;
		ao=clamp(ao,0.0,1.0);
		return ao;
	}

#endif


//==========================================
// Main
//==========================================

void main( void ) {
	
	//==========================================
	//General variables
	//==========================================	
	
	vec2 texcoord = gl_FragCoord.xy/buffersize;
	vec4 color;
	float depth;
	float lineardepth;
	vec3 normal;
	vec4 cnormal;
	vec4 outputcolor;	
	vec2 pixelsize = 1.0/buffersize;
	float irisadjustment=1.0;
	float bloomstrength = 1.0;
	float bloomirisadjustment=2.0;	
	
	#ifdef LW_WOBBLE
		Include "wobble.frag"
	#endif
	
	color = texture2D( texture0, texcoord ) * fragcolor;
	depth = texture2D( texture1, texcoord ).x;
	cnormal=texture2D( texture2, texcoord );
	normal = cnormal.xyz * 2.0 - 1.0;
	lineardepth = DepthToZPosition( depth );
	
	outputcolor = color;
	float pixelbrightness = outputcolor.r * 0.3 + outputcolor.g * 0.59 + outputcolor.b * 0.11;
	

	//==========================================
	// Now include the effects we want
	//==========================================
	
	#ifdef LW_HDR
		include "hdr.frag"
	#endif
	
	#ifdef LW_CAUSTICS
		include "caustics.frag"
	#endif

	#ifdef LW_MOTIONBLUR
		include "MotionBlur.frag"
	#endif
	
	#ifdef LW_BRIGHTEN
		outputcolor = max( outputcolor, texture2D(texture4,texcoord).w * texture2D(texture5,texcoord) );
	#endif

	#ifdef LW_NEARDOF
		Include "neardof.frag"
	#endif

	#ifdef LW_FARDOF
		Include "fardof.frag"
	#endif

	#ifdef LW_GODRAYS
		Include "godrays.frag"
	#endif
	
	#ifdef LW_SSAO
		float gray = outputcolor.r * 0.3 + outputcolor.g * 0.59 + outputcolor.b * 0.11;	
		gray = clamp((gray-0.25)/0.25,0.0,1.0);
		vec4 ssaocolor;
		ssaocolor =  texture2D(texture11,texcoord-pixelsize+vec2(-pixelsize.x*2.0,-pixelsize.y*2.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(0.0,-pixelsize.y*2.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(+pixelsize.x*2.0,-pixelsize.y*2.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(-pixelsize.x*2.0,0.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(0.0,0.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(+pixelsize.x*2.0,0.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(-pixelsize.x*2.0,+pixelsize.y));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(0.0,+pixelsize.y*2.0));
		ssaocolor += texture2D(texture11,texcoord-pixelsize+vec2(+pixelsize.x*2.0,+pixelsize.y*2.0));
		ssaocolor /= 9.0;
		ao = ssaocolor.w;
		outputcolor.rgb = mix(outputcolor.rgb*ao,outputcolor.rgb,gray);
		//outputcolor.x = outputcolor.rgb*0.5;
		//outputcolor.x += ssaocolor.x * 0.75;
		//outputcolor.y += ssaocolor.y * 0.75;
		//outputcolor.z += ssaocolor.z * 0.75;
		//outputcolor = vec4(ao);
		//outputcolor = vec4(ssaocolor.xyz*0.5,0);//outputcolor*0.75+vec4(ssaocolor.xyz,outputcolor.w)*0.25;
	#endif
	
	#ifdef LW_DISTANCEFOG
		Include "fog.frag"
	#endif

	#ifdef LW_BLOOM
		Include "bloom.frag"
	#endif
	
	#ifdef LW_GAUSSIAN
		Include "gaussian.frag"
	#endif
	
	#ifdef LW_BRIGHTNESS
	Include "brightness.frag"
	#endif
	#ifdef LW_CONTRAST
	Include "contrast.frag"
	#endif
	#ifdef LW_SATURATION
	Include "saturation.frag"
	#endif
	
	//==========================================
	// Set the fragment color to the output variable
	//==========================================
	
	gl_FragColor = outputcolor;
	gl_FragDepth = depth;
}